@@ -308,7 +308,7 @@ def rdrobust(y, x, c = None, fuzzy = None, deriv = None,
         weights = np.array(weights).reshape(-1,1)
         if subset is not None:
             weights = weights[subset]
-        na_ok = na_ok & complete_cases(weights)
+        na_ok = na_ok & complete_cases(weights) & weights>=0
     
     x = x[na_ok]
     y = y[na_ok]
